@@ -92,13 +92,6 @@ class ComputeCloudMixin(_normalize.Normalizer):
         flavors = self.list_flavors(get_extra=get_extra)
         return _utils._filter_list(flavors, name_or_id, filters)
 
-    def search_security_groups(self, name_or_id=None, filters=None):
-        # `filters` could be a dict or a jmespath (str)
-        groups = self.list_security_groups(
-            filters=filters if isinstance(filters, dict) else None
-        )
-        return _utils._filter_list(groups, name_or_id, filters)
-
     def search_servers(
             self, name_or_id=None, filters=None, detailed=False,
             all_projects=False, bare=False):
@@ -201,12 +194,11 @@ class ComputeCloudMixin(_normalize.Normalizer):
         if not self._has_secgroups():
             return []
 
-        data = proxy._json_response(
-            self.compute.get(
-                '/servers/{server_id}/os-security-groups'.format(
-                    server_id=server['id'])))
-        return self._normalize_secgroups(
-            self._get_and_munchify('security_groups', data))
+        server = self.compute.get_server(server)
+
+        server.fetch_security_groups(self.compute)
+
+        return self._normalize_secgroups(server.security_groups)
 
     def _get_server_security_groups(self, server, security_groups):
         if not self._has_secgroups():
@@ -258,9 +250,7 @@ class ComputeCloudMixin(_normalize.Normalizer):
             return False
 
         for sg in security_groups:
-            proxy._json_response(self.compute.post(
-                '/servers/%s/action' % server['id'],
-                json={'addSecurityGroup': {'name': sg.name}}))
+            self.compute.add_security_group_to_server(server, sg)
 
         return True
 
@@ -286,11 +276,9 @@ class ComputeCloudMixin(_normalize.Normalizer):
 
         for sg in security_groups:
             try:
-                proxy._json_response(self.compute.post(
-                    '/servers/%s/action' % server['id'],
-                    json={'removeSecurityGroup': {'name': sg.name}}))
+                self.compute.remove_security_group_from_server(server, sg)
 
-            except exc.OpenStackCloudURINotFound:
+            except exceptions.ResourceNotFound:
                 # NOTE(jamielennox): Is this ok? If we remove something that
                 # isn't present should we just conclude job done or is that an
                 # error? Nova returns ok if you try to add a group twice.
@@ -361,7 +349,8 @@ class ComputeCloudMixin(_normalize.Normalizer):
             # and the to_munch call.
             self._normalize_server(server._to_munch())
             for server in self.compute.servers(
-                all_projects=all_projects, **filters)]
+                all_projects=all_projects, allow_unknown_params=True,
+                **filters)]
         return [
             self._expand_server(server, detailed, bare)
             for server in servers
@@ -373,10 +362,7 @@ class ComputeCloudMixin(_normalize.Normalizer):
         :returns: A list of server group dicts.
 
         """
-        data = proxy._json_response(
-            self.compute.get('/os-server-groups'),
-            error_message="Error fetching server group list")
-        return self._get_and_munchify('server_groups', data)
+        return list(self.compute.server_groups())
 
     def get_compute_limits(self, name_or_id=None):
         """ Get compute limits for a project
@@ -519,10 +505,12 @@ class ComputeCloudMixin(_normalize.Normalizer):
             return ""
 
     def _get_server_console_output(self, server_id, length=None):
-        data = proxy._json_response(self.compute.post(
-            '/servers/{server_id}/action'.format(server_id=server_id),
-            json={'os-getConsoleOutput': {'length': length}}))
-        return self._get_and_munchify('output', data)
+        output = self.compute.get_server_console_output(
+            server=server_id,
+            length=length
+        )
+        if 'output' in output:
+            return output['output']
 
     def get_server(
             self, name_or_id=None, filters=None, detailed=False, bare=False,
@@ -676,40 +664,9 @@ class ComputeCloudMixin(_normalize.Normalizer):
                     "Server {server} could not be found and therefore"
                     " could not be snapshotted.".format(server=server))
             server = server_obj
-        response = proxy._json_response(
-            self.compute.post(
-                '/servers/{server_id}/action'.format(server_id=server['id']),
-                json={
-                    "createImage": {
-                        "name": name,
-                        "metadata": metadata,
-                    }
-                }))
-        # You won't believe it - wait, who am I kidding - of course you will!
-        # Nova returns the URL of the image created in the Location
-        # header of the response. (what?) But, even better, the URL it responds
-        # with has a very good chance of being wrong (it is built from
-        # nova.conf values that point to internal API servers in any cloud
-        # large enough to have both public and internal endpoints.
-        # However, nobody has ever noticed this because novaclient doesn't
-        # actually use that URL - it extracts the id from the end of
-        # the url, then returns the id. This leads us to question:
-        #   a) why Nova is going to return a value in a header
-        #   b) why it's going to return data that probably broken
-        #   c) indeed the very nature of the fabric of reality
-        # Although it fills us with existential dread, we have no choice but
-        # to follow suit like a lemming being forced over a cliff by evil
-        # producers from Disney.
-        # TODO(mordred) Update this to consume json microversion when it is
-        #               available.
-        #               blueprint:remove-create-image-location-header-response
-        image_id = response.headers['Location'].rsplit('/', 1)[1]
-        self.list_images.invalidate(self)
-        image = self.get_image(image_id)
-
-        if not wait:
-            return image
-        return self.wait_for_image(image, timeout=timeout)
+        image = self.compute.create_server_image(
+            server, name=name, metadata=metadata, wait=wait, timeout=timeout)
+        return image
 
     def get_server_id(self, name_or_id):
         server = self.get_server(name_or_id, bare=True)
@@ -1098,6 +1055,9 @@ class ComputeCloudMixin(_normalize.Normalizer):
         """
         Wait for a server to reach ACTIVE status.
         """
+        # server = self.compute.wait_for_server(
+        #     server=server, interval=self._SERVER_AGE or 2, wait=timeout
+        # )
         server_id = server['id']
         timeout_message = "Timeout waiting for the server to come up."
         start_time = time.time()
@@ -1236,11 +1196,7 @@ class ComputeCloudMixin(_normalize.Normalizer):
             raise exc.OpenStackCloudException(
                 'Invalid Server {server}'.format(server=name_or_id))
 
-        proxy._json_response(
-            self.compute.post(
-                '/servers/{server_id}/metadata'.format(server_id=server['id']),
-                json={'metadata': metadata}),
-            error_message='Error updating server metadata')
+        self.compute.set_server_metadata(server=server.id, **metadata)
 
     def delete_server_metadata(self, name_or_id, metadata_keys):
         """Delete metadata from a server instance.
@@ -1257,15 +1213,8 @@ class ComputeCloudMixin(_normalize.Normalizer):
             raise exc.OpenStackCloudException(
                 'Invalid Server {server}'.format(server=name_or_id))
 
-        for key in metadata_keys:
-            error_message = 'Error deleting metadata {key} on {server}'.format(
-                key=key, server=name_or_id)
-            proxy._json_response(
-                self.compute.delete(
-                    '/servers/{server_id}/metadata/{key}'.format(
-                        server_id=server['id'],
-                        key=key)),
-                error_message=error_message)
+        self.compute.delete_server_metadata(server=server.id,
+                                            keys=metadata_keys)
 
     def delete_server(
             self, name_or_id, wait=False, timeout=180, delete_ips=False,
@@ -1408,7 +1357,7 @@ class ComputeCloudMixin(_normalize.Normalizer):
             self._get_and_munchify('server', data))
         return self._expand_server(server, bare=bare, detailed=detailed)
 
-    def create_server_group(self, name, policies):
+    def create_server_group(self, name, policies=[], policy=None):
         """Create a new server group.
 
         :param name: Name of the server group being created
@@ -1418,16 +1367,16 @@ class ComputeCloudMixin(_normalize.Normalizer):
 
         :raises: OpenStackCloudException on operation error.
         """
-        data = proxy._json_response(
-            self.compute.post(
-                '/os-server-groups',
-                json={
-                    'server_group': {
-                        'name': name,
-                        'policies': policies}}),
-            error_message="Unable to create server group {name}".format(
-                name=name))
-        return self._get_and_munchify('server_group', data)
+        sg_attrs = {
+            'name': name
+        }
+        if policies:
+            sg_attrs['policies'] = policies
+        if policy:
+            sg_attrs['policy'] = policy
+        return self.compute.create_server_group(
+            **sg_attrs
+        )
 
     def delete_server_group(self, name_or_id):
         """Delete a server group.
@@ -1444,12 +1393,7 @@ class ComputeCloudMixin(_normalize.Normalizer):
                            name_or_id)
             return False
 
-        proxy._json_response(
-            self.compute.delete(
-                '/os-server-groups/{id}'.format(id=server_group['id'])),
-            error_message="Error deleting server group {name}".format(
-                name=name_or_id))
-
+        self.compute.delete_server_group(server_group, ignore_missing=False)
         return True
 
     def create_flavor(self, name, ram, vcpus, disk, flavorid="auto",
@@ -1597,16 +1541,16 @@ class ComputeCloudMixin(_normalize.Normalizer):
         return _utils.normalize_flavor_accesses(
             self._get_and_munchify('flavor_access', data))
 
-    def list_hypervisors(self):
+    def list_hypervisors(self, filters={}):
         """List all hypervisors
 
         :returns: A list of hypervisor ``munch.Munch``.
         """
 
-        data = proxy._json_response(
-            self.compute.get('/os-hypervisors/detail'),
-            error_message="Error fetching hypervisor list")
-        return self._get_and_munchify('hypervisors', data)
+        return list(self.compute.hypervisors(
+            details=True,
+            allow_unknown_params=True,
+            **filters))
 
     def search_aggregates(self, name_or_id=None, filters=None):
         """Seach host aggregates.
