@@ -9,6 +9,7 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
+import warnings
 
 from openstack.compute.v2 import aggregate as _aggregate
 from openstack.compute.v2 import availability_zone
@@ -25,6 +26,7 @@ from openstack.compute.v2 import server_interface as _server_interface
 from openstack.compute.v2 import server_ip
 from openstack.compute.v2 import service as _service
 from openstack.compute.v2 import volume_attachment as _volume_attachment
+from openstack.network.v2 import security_group as _sg
 from openstack import proxy
 from openstack import resource
 
@@ -275,16 +277,18 @@ class Proxy(proxy.Proxy):
         """Return a generator of images
 
         :param bool details: When ``True``, returns
-            :class:`~openstack.compute.v2.image.ImageDetail` objects,
-            otherwise :class:`~openstack.compute.v2.image.Image`.
+            :class:`~openstack.compute.v2.image.Image` objects with all
+            available properties, otherwise only basic properties are returned.
             *Default: ``True``*
         :param kwargs query: Optional query parameters to be sent to limit
                                  the resources being returned.
 
         :returns: A generator of image objects
         """
-        img = _image.ImageDetail if details else _image.Image
-        return self._list(img, **query)
+        warnings.warn('This API is deprecated and may disappear shortly',
+                      DeprecationWarning)
+        base_path = '/images/detail' if details else None
+        return self._list(_image.Image, base_path=base_path, **query)
 
     def _get_base_resource(self, res, base):
         # Metadata calls for Image and Server can work for both those
@@ -299,9 +303,7 @@ class Proxy(proxy.Proxy):
         """Return a dictionary of metadata for an image
 
         :param image: Either the ID of an image or a
-                       :class:`~openstack.compute.v2.image.Image` or
-                       :class:`~openstack.compute.v2.image.ImageDetail`
-                       instance.
+            :class:`~openstack.compute.v2.image.Image` instance.
 
         :returns: A :class:`~openstack.compute.v2.image.Image` with only the
                   image's metadata. All keys and values are Unicode text.
@@ -316,9 +318,7 @@ class Proxy(proxy.Proxy):
         """Update metadata for an image
 
         :param image: Either the ID of an image or a
-                       :class:`~openstack.compute.v2.image.Image` or
-                       :class:`~openstack.compute.v2.image.ImageDetail`
-                       instance.
+            :class:`~openstack.compute.v2.image.Image` instance.
         :param kwargs metadata: Key/value pairs to be updated in the image's
                                 metadata. No other metadata is modified
                                 by this call. All keys and values are stored
@@ -339,9 +339,7 @@ class Proxy(proxy.Proxy):
         Note: This method will do a HTTP DELETE request for every key in keys.
 
         :param image: Either the ID of an image or a
-                       :class:`~openstack.compute.v2.image.Image` or
-                       :class:`~openstack.compute.v2.image.ImageDetail`
-                       instance.
+            :class:`~openstack.compute.v2.image.Image` instance.
         :param keys: The keys to delete.
 
         :rtype: ``None``
@@ -624,39 +622,58 @@ class Proxy(proxy.Proxy):
         server = self._get_resource(_server.Server, server)
         server.revert_resize(self)
 
-    def create_server_image(self, server, name, metadata=None):
+    def create_server_image(self, server, name, metadata=None, wait=False,
+                            timeout=120):
         """Create an image from a server
 
         :param server: Either the ID of a server or a
-                       :class:`~openstack.compute.v2.server.Server` instance.
+            :class:`~openstack.compute.v2.server.Server` instance.
         :param str name: The name of the image to be created.
         :param dict metadata: A dictionary of metadata to be set on the image.
 
-        :returns: None
+        :returns: :class:`~openstack.image.v2.image.Image` object.
         """
         server = self._get_resource(_server.Server, server)
-        server.create_image(self, name, metadata)
+        image_id = server.create_image(self, name, metadata)
+
+        self._connection.list_images.invalidate(self)
+        image = self._connection.get_image(image_id)
+
+        if not wait:
+            return image
+        return self._connection.wait_for_image(image, timeout=timeout)
+
+    def fetch_server_security_groups(self, server):
+        """Fetch security groups with details for a server.
+
+        :param server: Either the ID of a server or a
+            :class:`~openstack.compute.v2.server.Server` instance.
+
+        :returns: updated :class:`~openstack.compute.v2.server.Server` instance
+        """
+        server = self._get_resource(_server.Server, server)
+        return server.fetch_security_groups(self)
 
     def add_security_group_to_server(self, server, security_group):
         """Add a security group to a server
 
         :param server: Either the ID of a server or a
-                       :class:`~openstack.compute.v2.server.Server` instance.
-        :param security_group: Either the ID of a security group or a
+            :class:`~openstack.compute.v2.server.Server` instance.
+        :param security_group: Either the ID, Name of a security group or a
             :class:`~openstack.network.v2.security_group.SecurityGroup`
             instance.
 
         :returns: None
         """
         server = self._get_resource(_server.Server, server)
-        security_group_id = resource.Resource._get_id(security_group)
-        server.add_security_group(self, security_group_id)
+        security_group = self._get_resource(_sg.SecurityGroup, security_group)
+        server.add_security_group(self, security_group.name)
 
     def remove_security_group_from_server(self, server, security_group):
         """Remove a security group from a server
 
         :param server: Either the ID of a server or a
-                       :class:`~openstack.compute.v2.server.Server` instance.
+            :class:`~openstack.compute.v2.server.Server` instance.
         :param security_group: Either the ID of a security group or a
             :class:`~openstack.network.v2.security_group.SecurityGroup`
             instance.
@@ -664,8 +681,8 @@ class Proxy(proxy.Proxy):
         :returns: None
         """
         server = self._get_resource(_server.Server, server)
-        security_group_id = resource.Resource._get_id(security_group)
-        server.remove_security_group(self, security_group_id)
+        security_group = self._get_resource(_sg.SecurityGroup, security_group)
+        server.remove_security_group(self, security_group.name)
 
     def add_fixed_ip_to_server(self, server, network_id):
         """Adds a fixed IP address to a server instance.
@@ -1068,9 +1085,7 @@ class Proxy(proxy.Proxy):
         """Update metadata for a server
 
         :param server: Either the ID of a server or a
-                       :class:`~openstack.compute.v2.server.Server` or
-                       :class:`~openstack.compute.v2.server.ServerDetail`
-                       instance.
+            :class:`~openstack.compute.v2.server.Server` instance.
         :param kwargs metadata: Key/value pairs to be updated in the server's
                                 metadata. No other metadata is modified
                                 by this call. All keys and values are stored
@@ -1091,9 +1106,7 @@ class Proxy(proxy.Proxy):
         Note: This method will do a HTTP DELETE request for every key in keys.
 
         :param server: Either the ID of a server or a
-                       :class:`~openstack.compute.v2.server.Server` or
-                       :class:`~openstack.compute.v2.server.ServerDetail`
-                       instance.
+            :class:`~openstack.compute.v2.server.Server` instance.
         :param keys: The keys to delete
 
         :rtype: ``None``
@@ -1171,23 +1184,19 @@ class Proxy(proxy.Proxy):
         """
         return self._list(_server_group.ServerGroup, **query)
 
-    def hypervisors(self, details=False):
+    def hypervisors(self, details=False, **query):
         """Return a generator of hypervisor
 
         :param bool details: When set to the default, ``False``,
-                    :class:`~openstack.compute.v2.hypervisor.Hypervisor`
-                    instances will be returned. ``True`` will cause
-                    :class:`~openstack.compute.v2.hypervisor.HypervisorDetail`
-                    instances to be returned.
+            :class:`~openstack.compute.v2.hypervisor.Hypervisor`
+            instances will be returned with only basic information populated.
+        :param kwargs query: Optional query parameters to be sent to limit
+            the resources being returned.
         :returns: A generator of hypervisor
         :rtype: class: `~openstack.compute.v2.hypervisor.Hypervisor`
         """
-        if details:
-            hypervisor = _hypervisor.HypervisorDetail
-        else:
-            hypervisor = _hypervisor.Hypervisor
-
-        return self._list(hypervisor)
+        base_path = '/os-hypervisors/detail' if details else None
+        return self._list(_hypervisor.Hypervisor, base_path=base_path, **query)
 
     def find_hypervisor(self, name_or_id, ignore_missing=True):
         """Find a hypervisor from name or id to get the corresponding info
