@@ -9,14 +9,17 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
-
+import hashlib
 import operator
+import six
+import tempfile
 
 from keystoneauth1 import adapter
 import mock
 import requests
 from openstack.tests.unit import base
 
+from openstack import _log
 from openstack import exceptions
 from openstack.image.v2 import image
 
@@ -50,7 +53,7 @@ EXAMPLE = {
     'url': '20',
     'metadata': {'21': '22'},
     'architecture': '23',
-    'hypervisor-type': '24',
+    'hypervisor_type': '24',
     'instance_type_rxtx_factor': 25.1,
     'instance_uuid': '26',
     'img_config_drive': '27',
@@ -85,10 +88,18 @@ EXAMPLE = {
 }
 
 
+def calculate_md5_checksum(data):
+    checksum = hashlib.md5()
+    for chunk in data:
+        checksum.update(chunk)
+    return checksum.hexdigest()
+
+
 class FakeResponse(object):
     def __init__(self, response, status_code=200, headers=None, reason=None):
         self.body = response
         self.content = response
+        self.text = response
         self.status_code = status_code
         headers = headers if headers else {'content-type': 'application/json'}
         self.headers = requests.structures.CaseInsensitiveDict(headers)
@@ -112,9 +123,10 @@ class TestImage(base.TestCase):
         self.sess.post = mock.Mock(return_value=self.resp)
         self.sess.put = mock.Mock(return_value=FakeResponse({}))
         self.sess.delete = mock.Mock(return_value=FakeResponse({}))
-        self.sess.fetch = mock.Mock(return_value=FakeResponse({}))
+        self.sess.get = mock.Mock(return_value=FakeResponse({}))
         self.sess.default_microversion = None
         self.sess.retriable_status_codes = None
+        self.sess.log = _log.setup_logging('openstack')
 
     def test_basic(self):
         sot = image.Image()
@@ -175,7 +187,7 @@ class TestImage(base.TestCase):
         self.assertEqual(EXAMPLE['url'], sot.url)
         self.assertEqual(EXAMPLE['metadata'], sot.metadata)
         self.assertEqual(EXAMPLE['architecture'], sot.architecture)
-        self.assertEqual(EXAMPLE['hypervisor-type'], sot.hypervisor_type)
+        self.assertEqual(EXAMPLE['hypervisor_type'], sot.hypervisor_type)
         self.assertEqual(EXAMPLE['instance_type_rxtx_factor'],
                          sot.instance_type_rxtx_factor)
         self.assertEqual(EXAMPLE['instance_uuid'], sot.instance_uuid)
@@ -255,22 +267,39 @@ class TestImage(base.TestCase):
 
     def test_import_image_with_uri_not_web_download(self):
         sot = image.Image(**EXAMPLE)
-        self.assertRaises(exceptions.InvalidRequest,
-                          sot.import_image,
-                          self.sess,
-                          "glance-direct",
-                          "such-a-good-uri")
+
+        sot.import_image(self.sess, "glance-direct")
+        self.sess.post.assert_called_with(
+            'images/IDENTIFIER/import',
+            json={"method": {"name": "glance-direct"}}
+        )
 
     def test_upload(self):
         sot = image.Image(**EXAMPLE)
 
-        self.assertIsNone(sot.upload(self.sess))
+        self.assertIsNotNone(sot.upload(self.sess))
         self.sess.put.assert_called_with('images/IDENTIFIER/file',
                                          data=sot.data,
                                          headers={"Content-Type":
                                                   "application/octet-stream",
                                                   "Accept": ""})
 
+    def test_stage(self):
+        sot = image.Image(**EXAMPLE)
+
+        self.assertIsNotNone(sot.stage(self.sess))
+        self.sess.put.assert_called_with('images/IDENTIFIER/stage',
+                                         data=sot.data,
+                                         headers={"Content-Type":
+                                                  "application/octet-stream",
+                                                  "Accept": ""})
+
+    def test_stage_error(self):
+        sot = image.Image(**EXAMPLE)
+
+        self.sess.put.return_value = FakeResponse("dummy", status_code=400)
+        self.assertRaises(exceptions.SDKException, sot.stage, self.sess)
+
     def test_download_checksum_match(self):
         sot = image.Image(**EXAMPLE)
 
@@ -284,7 +313,7 @@ class TestImage(base.TestCase):
         self.sess.get.assert_called_with('images/IDENTIFIER/file',
                                          stream=False)
 
-        self.assertEqual(rv, resp.content)
+        self.assertEqual(rv, resp)
 
     def test_download_checksum_mismatch(self):
         sot = image.Image(**EXAMPLE)
@@ -314,7 +343,7 @@ class TestImage(base.TestCase):
                        stream=False),
              mock.call('images/IDENTIFIER', microversion=None)])
 
-        self.assertEqual(rv, resp1.content)
+        self.assertEqual(rv, resp1)
 
     def test_download_no_checksum_at_all2(self):
         sot = image.Image(**EXAMPLE)
@@ -332,15 +361,18 @@ class TestImage(base.TestCase):
             self.assertEqual(len(log.records), 1,
                              "Too many warnings were logged")
             self.assertEqual(
-                "Unable to verify the integrity of image IDENTIFIER",
+                "Unable to verify the integrity of image %s",
                 log.records[0].msg)
+            self.assertEqual(
+                (sot.id,),
+                log.records[0].args)
 
         self.sess.get.assert_has_calls(
             [mock.call('images/IDENTIFIER/file',
                        stream=False),
              mock.call('images/IDENTIFIER', microversion=None)])
 
-        self.assertEqual(rv, resp1.content)
+        self.assertEqual(rv, resp1)
 
     def test_download_stream(self):
         sot = image.Image(**EXAMPLE)
@@ -356,6 +388,37 @@ class TestImage(base.TestCase):
 
         self.assertEqual(rv, resp)
 
+    def test_image_download_output_fd(self):
+        output_file = six.BytesIO()
+        sot = image.Image(**EXAMPLE)
+        response = mock.Mock()
+        response.status_code = 200
+        response.iter_content.return_value = [b'01', b'02']
+        response.headers = {
+            'Content-MD5':
+            calculate_md5_checksum(response.iter_content.return_value)
+        }
+        self.sess.get = mock.Mock(return_value=response)
+        sot.download(self.sess, output=output_file)
+        output_file.seek(0)
+        self.assertEqual(b'0102', output_file.read())
+
+    def test_image_download_output_file(self):
+        sot = image.Image(**EXAMPLE)
+        response = mock.Mock()
+        response.status_code = 200
+        response.iter_content.return_value = [b'01', b'02']
+        response.headers = {
+            'Content-MD5':
+            calculate_md5_checksum(response.iter_content.return_value)
+        }
+        self.sess.get = mock.Mock(return_value=response)
+
+        output_file = tempfile.NamedTemporaryFile()
+        sot.download(self.sess, output=output_file.name)
+        output_file.seek(0)
+        self.assertEqual(b'0102', output_file.read())
+
     def test_image_update(self):
         values = EXAMPLE.copy()
         del values['instance_uuid']
